"""Parser module will process and aggregate job posting files.
"""
import logging
import os
import re
from configparser import NoOptionError, NoSectionError
from dataclasses import dataclass, field

from bs4 import BeautifulSoup
from pandas import DataFrame

from parsers.parser_base import Parser


class NoImportFiles(Exception):
    pass


class InvalidConfigArg(Exception):
    pass


class InvalidFileName(Exception):
    pass


class EmptySoup(Exception):
    "That is some bad soup"
    pass


@dataclass
class Posting:
    # config props
    input_file_name: str
    input_file: str = ""
    soup: BeautifulSoup = field(default=BeautifulSoup(), repr=False)

    # export props
    jobid: str = ""
    url: str = ""
    title: str = ""
    workplace_type: str = ""
    company_name: str = ""
    company_url: str = ""
    company_size: str = ""
    company_industry: str = ""
    hours: str = ""
    level: str = ""

    # used to replace asdict() as that has limited options
    def to_dict(self) -> dict:
        return {
            "jobid": self.jobid,
            "url": self.url,
            "title": self.title,
            "company_name": self.company_name,
            "workplace_type": self.workplace_type,
            "company_url": self.company_url,
            "company_size": self.company_size,
            "company_industry": self.company_industry,
            "hours": self.hours,
            "level": self.level,
        }


class LinkedinParser(Parser):
    def __init__(self, config):
        self.config = config

        # lists to hold each posting
        self.postings: list[dict] = []
        self.postings_err: list[dict] = []

        try:
            self.output_file = config.get("Parser", "output_file")
            self.output_file_errors = config.get("Parser", "output_file_err")
            self.input_dir = config.get("Parser", "input_dir")
        except NoSectionError as e:
            raise InvalidConfigArg(e.message) from None
        except NoOptionError as e:
            raise InvalidConfigArg(e.message) from None

    # parser
    def parse_export(self) -> None:
        """Export all postings to CSV file"""
        logging.info(f"Exporting to {self.output_file}")
        DataFrame(self.postings).to_csv(self.output_file, index=False)

    # parser
    def parse_postings(self) -> None:

        if len(os.listdir(self.input_dir)) == 0:
            raise NoImportFiles("There are no files to import")

        for input_file_name in os.listdir(self.input_dir):

            post = Posting(input_file_name)

            # posting - set config props
            post.input_file = self.set_posting_input_file(
                self.input_dir, input_file_name
            )
            post.soup = self.set_posting_soup(post.input_file)

            # posting - set export props
            post.jobid = self.set_posting_jobid(input_file_name)
            post.url = self.set_posting_url(post.jobid)
            post.title = self.set_posting_title(post.soup)
            post.workplace_type = self.set_posting_workplace_type(post.soup)
            post.company_name = self.set_posting_company_name(post.soup)
            post.company_url = self.set_posting_company_url(post.soup)
            (
                post.company_size,
                post.company_industry,
                post.hours,
                post.level,
            ) = self.set_posting_company_details(post.soup)

            self.postings.append(post.to_dict())

        self.parse_export()

    # input_file - config prop
    def set_posting_input_file(
        self, input_dir: str, input_file_name: str
    ) -> str:
        return os.path.join(input_dir, input_file_name)

    # soup - config prop
    def set_posting_soup(self, input_file: str) -> BeautifulSoup:

        try:
            with open(input_file, mode="r", encoding="UTF-8") as f:
                soup = BeautifulSoup(f, "html.parser")
        except FileNotFoundError as e:
            raise InvalidFileName(e) from None
        except OSError as e:
            raise InvalidFileName(e) from None
        else:
            return soup

    # jobid - export prop
    def set_posting_jobid(self, input_file_name: str) -> str:
        try:
            jobid = input_file_name.split("_")[1]
        except IndexError as e:
            raise InvalidFileName(e) from None
        else:
            logging.info(f"{jobid} - Parsing job ")
            return jobid

    # url - export prop
    def set_posting_url(self, jobid: str) -> str:
        return "https://www.linkedin.com/jobs/view/" + jobid

    # title - export prop
    def set_posting_title(self, soup: BeautifulSoup) -> str:

        try:
            assert type(soup) is BeautifulSoup
            title = soup.find(class_="t-24")

            if title is None:
                raise EmptySoup("workplace_type is missing")

        except EmptySoup as e:
            logging.error(f"Err msg - {e}")
            return "missing"
        except AssertionError:
            logging.error(f"Soup should be BeautifulSoup, not {type(soup)}")
            return "error"
        else:
            return title.text.strip()

    # workplace_type - export prop
    def set_posting_workplace_type(self, soup: BeautifulSoup) -> str:

        try:
            assert type(soup) is BeautifulSoup

            # workplace_type. looking for remote (f_WT=2 in url)
            workplace_type = soup.find(
                class_="jobs-unified-top-card__workplace-type"
            )

            if workplace_type is None:
                raise EmptySoup("workplace_type is missing")

        except EmptySoup as e:
            logging.error(f"Err msg - {e}")
            return "missing"
        except AssertionError:
            logging.error(f"Soup should be BeautifulSoup, not {type(soup)}")
            return "error"
        else:
            return workplace_type.text.strip()

    # company_name - export prop
    def set_posting_company_name(self, soup: BeautifulSoup) -> str:

        try:
            assert type(soup) is BeautifulSoup
            company_name = soup.find(
                "span", class_="jobs-unified-top-card__company-name"
            ).find("a")

        # AttributeError can occur on second find()
        except AttributeError as e:
            logging.error(f"Err msg - {e}")
            return "missing"
        except AssertionError:
            logging.error(f"Soup should be BeautifulSoup, not {type(soup)}")
            return "error"
        else:
            return company_name.text.strip()

    # company_url - export prop
    def set_posting_company_url(self, soup: BeautifulSoup) -> str:

        try:
            assert type(soup) is BeautifulSoup
            company_url = soup.find(
                "span", class_="jobs-unified-top-card__company-name"
            ).find("a")["href"]

        # AttributeError can occur on second find()
        except AttributeError as e:
            logging.error(f"Err msg - {e}")
            return "missing"
        except AssertionError:
            logging.error(f"Soup should be BeautifulSoup, not {type(soup)}")
            return "error"
        else:
            return company_url.strip()

    # company_details - export props
    def set_posting_company_details(self, soup: BeautifulSoup) -> tuple:

        try:
            assert type(soup) is BeautifulSoup
            company_details = soup.find_all(string=re.compile(r" · "))
        except AssertionError:
            logging.error(f"Soup should be BeautifulSoup, not {type(soup)}")
            return "error", "error", "error", "error"

        company_size: str = "missing"
        company_industry: str = "missing"
        hours: str = "missing"
        level: str = "missing"

        for section in company_details:

            section = section.strip()

            # remove leading dots if they exist
            if section[0] == "·":
                section = section[1:].strip()

            if "employees" in section and section.count(" · ") == 1:
                company_size, company_industry = section.split(" · ")
            elif "Full-time" in section and section.count(" · ") == 1:
                hours, level = section.split(" · ")

        return company_size, company_industry, hours, level



"""exfill will scrape LinkedIn job postings, parse out details about
each posting, then combine all of the information into a single useable
csv file.
"""
import logging
import os
from argparse import ArgumentParser
from configparser import ConfigParser, ExtendedInterpolation

from parsers.parser_factory import ParserFactory
from scrapers.factory import ScraperFactory


class ConfigFileMissing(Exception):
    pass


def init_parser() -> dict:
    """Initialize argument parser."""
    parser = ArgumentParser()
    parser.add_argument("site", choices=["linkedin"], help="Site to scrape")
    parser.add_argument(
        "action", choices=["scrape", "parse"], help="Action to perform"
    )
    return vars(parser.parse_args())


def load_config() -> ConfigParser:
    """Load config file"""

    config_file = os.path.dirname(__file__) + "/config.ini"
    if not os.path.exists(config_file):
        raise ConfigFileMissing("Default config.ini is missing")

    config = ConfigParser(interpolation=ExtendedInterpolation())
    config.read(config_file)

    return config


def create_dirs(config: ConfigParser) -> None:
    """Create directories referenced in the config file"""
    # for item in config['Directories']:
    for dir_path in config.items("Directories"):
        if not os.path.exists(dir_path[1]):
            os.mkdir(dir_path[1])


def main() -> None:
    """Main controller function used to call child functions/modules."""
    # Load config
    config = load_config()

    create_dirs(config)

    # Initialize logging
    log_file_name = config.get("Paths", "app_log")
    logging.basicConfig(
        filename=log_file_name,
        level=logging.INFO,  # level=logging.INFO should be default
        format="[%(asctime)s] [%(levelname)s] - %(message)s",
        filemode="w+",
    )

    args = init_parser()
    logging.info(f"Starting app with the following input args: {args}")

    if args.get("action") == "scrape":
        scraper = ScraperFactory.create("linkedin", config)
        scraper.scrape_postings(48)

    if args.get("action") == "parse":
        parser = ParserFactory.create("linkedin", config)
        parser.parse_postings()

    logging.info("Finished execution.  Exiting application.")


if __name__ == "__main__":

    main()



from bs4 import BeautifulSoup

from parsers.linkedin_parser import InvalidFileName, LinkedinParser
from tests.main import BaseTest


class TestLinkedinParserPosting(BaseTest):
    def setUp(self) -> None:

        self.parser = LinkedinParser(BaseTest.load_test_config(self))

        # good_sip - defined in the test
        self.empty_sip = BeautifulSoup("", "html.parser")  # returns 'missing'
        self.bad_sip = "no soup for you"  # returns 'error'

    # jobid - export props
    def test_set_posting_jobid(self):

        good_file = "jobid_3080721373_20220516_180204.html"
        jobid = self.parser.set_posting_jobid(good_file)

        self.assertEqual(jobid, "3080721373")
        self.assertIsInstance(jobid, str)

        with self.assertRaises(TypeError):
            self.parser.set_posting_jobid()

        with self.assertRaises(InvalidFileName):
            self.parser.set_posting_jobid("")

        with self.assertRaises(InvalidFileName):
            self.parser.set_posting_jobid("nounderscores")

    # url - export prop
    def test_set_posting_url(self):

        url = self.parser.set_posting_url("3080721373")

        self.assertEqual(url, "https://www.linkedin.com/jobs/view/3080721373")
        self.assertIsInstance(url, str)

        with self.assertRaises(TypeError):
            self.parser.set_posting_url()

    # title - export prop
    def test_set_posting_title(self):

        good_sip = BeautifulSoup(
            ('<h2 class="t-24 t-bold">' "Senior DevOps Engineer" "</h2>"),
            "html.parser",
        )

        title = self.parser.set_posting_title(good_sip)
        self.assertEqual(title, "Senior DevOps Engineer")
        self.assertIsInstance(title, str)

        self.assertEqual(
            self.parser.set_posting_title(self.empty_sip), "missing"
        )

        self.assertEqual(self.parser.set_posting_title(self.bad_sip), "error")

        with self.assertRaises(TypeError):
            self.parser.set_posting_title()

    # workplace_type - export prop
    def test_set_posting_workplace_type(self):

        good_sip = BeautifulSoup(
            (
                '<span class="jobs-unified-top-card__workplace-type">'
                "Remote"
                "</span>"
            ),
            "html.parser",
        )

        workingplace_type = self.parser.set_posting_workplace_type(good_sip)
        self.assertEqual(workingplace_type, "Remote")
        self.assertIsInstance(workingplace_type, str)

        self.assertEqual(
            self.parser.set_posting_workplace_type(self.empty_sip), "missing"
        )

        self.assertEqual(
            self.parser.set_posting_workplace_type(self.bad_sip), "error"
        )

        with self.assertRaises(TypeError):
            self.parser.set_posting_workplace_type()

    # company_name - export prop
    def test_set_posting_company_name(self):

        good_sip = BeautifulSoup(
            (
                '<span class="jobs-unified-top-card__company-name">'
                '<a class="ember-view t-black t-normal"'
                'href="/company/sap/life/" id="ember146">'
                "SAP"
                "</a>"
                "</span>"
            ),
            "html.parser",
        )

        title = self.parser.set_posting_company_name(good_sip)
        self.assertEqual(title, "SAP")
        self.assertIsInstance(title, str)

        self.assertEqual(
            self.parser.set_posting_company_name(self.empty_sip), "missing"
        )

        self.assertEqual(
            self.parser.set_posting_company_name(self.bad_sip), "error"
        )

        with self.assertRaises(TypeError):
            self.parser.set_posting_company_name()

    # company_url - export prop
    def test_set_posting_company_url(self):

        good_sip = BeautifulSoup(
            (
                '<span class="jobs-unified-top-card__company-name">'
                '<a class="ember-view t-black t-normal"'
                'href="/company/sap/life/" id="ember146">'
                "SAP"
                "</a>"
                "</span>"
            ),
            "html.parser",
        )

        url = self.parser.set_posting_company_url(good_sip)
        self.assertEqual(url, "/company/sap/life/")
        self.assertIsInstance(url, str)

        with self.assertRaises(TypeError):
            self.parser.set_posting_company_url()

        self.assertEqual(
            self.parser.set_posting_company_url(self.empty_sip), "missing"
        )

        self.assertEqual(
            self.parser.set_posting_company_url(self.bad_sip), "error"
        )

    # company_details - export props
    def test_set_posting_company_details(self):

        good_sip = BeautifulSoup(
            (
                "<span>"
                '<a class="app-aware-link" href="#SALARY" target="_self">'
                "<!-- -->"
                "$80,000/yr - $125,000/yr"
                "<!-- -->"
                "</a>"
                '<span class="white-space-pre">'
                "</span>"
                " · Full-time · Mid-Senior level"
                "<!-- -->"
                "</span>"
                "<span>"
                "<!-- -->"
                "51-200 employees · Internet Publishing "
                "<!-- -->"
                "</span>"
            ),
            "html.parser",
        )

        (
            company_size,
            company_industry,
            hours,
            level,
        ) = self.parser.set_posting_company_details(good_sip)

        self.assertEqual(company_size, "51-200 employees")
        self.assertEqual(company_industry, "Internet Publishing")
        self.assertEqual(hours, "Full-time")
        self.assertEqual(level, "Mid-Senior level")

        self.assertIsInstance(company_size, str)
        self.assertIsInstance(company_industry, str)
        self.assertIsInstance(hours, str)
        self.assertIsInstance(level, str)

        with self.assertRaises(TypeError):
            self.parser.set_posting_company_details()

        self.assertEqual(
            self.parser.set_posting_company_details(self.empty_sip),
            ("missing", "missing", "missing", "missing"),
        )

        self.assertEqual(
            self.parser.set_posting_company_details(self.bad_sip),
            ("error", "error", "error", "error"),
        )

        dirty_sip = BeautifulSoup(" · one  · two  · three  · ", "html.parser")
        self.assertEqual(
            self.parser.set_posting_company_details(dirty_sip),
            ("missing", "missing", "missing", "missing"),
        )



from parsers.linkedin_parser import LinkedinParser
from parsers.parser_factory import NoMatchingParserType, ParserFactory
from tests.main import BaseTest


class TestParserFactory(BaseTest):
    def test_parser_factory_type(self):
        self.config = BaseTest.load_test_config(self)

        self.assertIsInstance(
            ParserFactory.create("linkedin", self.config), LinkedinParser
        )

        with self.assertRaises(NoMatchingParserType):
            ParserFactory.create("not_linkedin", self.config)



import os
import unittest
from configparser import ConfigParser, ExtendedInterpolation
from pathlib import Path

CURRENT_DIR = Path(__file__).parent


class BaseTest(unittest.TestCase):
    @staticmethod
    def load_test_config(self):
        config_file = os.path.abspath(os.curdir) + "/src/exfill/config.ini"
        self.config = ConfigParser(interpolation=ExtendedInterpolation())
        self.config.read(config_file)
        return self.config

    def load_test_data_html_path(self):
        return CURRENT_DIR / "test_data/html"

    # def test_print_config(self):
    #     for dir_path in self.config.sections():
    #         print(dir_path)
    #     assert 1


if __name__ == "__main__":
    unittest.main()



from configparser import ConfigParser

from extractor import load_config
from tests.main import BaseTest


class TestExtractor(BaseTest):
    def test_loading_config(self):
        config = load_config()
        self.assertIsInstance(config, ConfigParser)
